#include <konkret/konkret.h>
#include "Linux_AccountCapabilities.h"
#include "Linux_Account.h"
#include "Linux_EnabledAccountCapabilities.h"

#include "globals.h"
#include "aux_lu.h"

#include <libuser/entity.h>
#include <libuser/user.h>

#include <glib.h>

static const CMPIBroker* _cb;

static void Linux_AccountCapabilitiesInitialize()
{
}

static CMPIStatus Linux_AccountCapabilitiesCleanup( 
    CMPIInstanceMI* mi,
    const CMPIContext* cc, 
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Linux_AccountCapabilitiesEnumInstanceNames( 
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus Linux_AccountCapabilitiesEnumInstances( 
    CMPIInstanceMI* mi,
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const char** properties) 
{
    Linux_AccountRef laref;
    Linux_EnabledAccountCapabilitiesRef leacref;
    Linux_AccountCapabilities lac;

    struct lu_context *luc = NULL;
    struct lu_error *error = NULL;
    GPtrArray *accounts = NULL;
    struct lu_ent *lue = NULL;

    size_t i;
    const char *nameSpace = KNameSpace(cop);
    const char *hostname = get_system_name();

    Linux_EnabledAccountCapabilitiesRef_Init(&leacref, _cb, nameSpace);
    Linux_EnabledAccountCapabilitiesRef_Set_InstanceID(&leacref,
      ORGID":"LEACNAME);

    luc = lu_start(NULL, lu_user, NULL, NULL, lu_prompt_console_quiet, NULL, &error);
    if (!luc)
      {
        fprintf(stderr, "Error initializing: %s\n", lu_strerror(error));
        exit(1);
      }
    accounts = lu_users_enumerate_full(luc, "*", &error);

    for (i = 0;  (accounts != NULL) && (i < accounts->len); i++)
      {
        lue = g_ptr_array_index(accounts, i);

        Linux_AccountRef_Init(&laref, _cb, nameSpace);
        Linux_AccountRef_Set_Name(&laref, aux_lu_get_str(lue, LU_USERNAME));
        Linux_AccountRef_Set_SystemCreationClassName(&laref, SCCLASSNAME);
        Linux_AccountRef_Set_SystemName(&laref, hostname);
        Linux_AccountRef_Set_CreationClassName(&laref,
          Linux_Account_ClassName);

        Linux_AccountCapabilities_Init(&lac, _cb, nameSpace);
        Linux_AccountCapabilities_Set_ManagedElement(&lac, &laref);
        Linux_AccountCapabilities_Set_Capabilities(&lac, &leacref);

        KReturnInstance(cr, lac);
        lu_ent_free(lue);
      } /* for */

    if (accounts)
      {
        g_ptr_array_free(accounts, TRUE);
      }

    lu_end(luc);

    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Linux_AccountCapabilitiesGetInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc,
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const char** properties) 
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus Linux_AccountCapabilitiesCreateInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const CMPIInstance* ci) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_AccountCapabilitiesModifyInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop,
    const CMPIInstance* ci, 
    const char**properties) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_AccountCapabilitiesDeleteInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_AccountCapabilitiesExecQuery(
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const char* lang, 
    const char* query) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_AccountCapabilitiesAssociationCleanup( 
    CMPIAssociationMI* mi,
    const CMPIContext* cc, 
    CMPIBoolean term) 
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Linux_AccountCapabilitiesAssociators(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole,
    const char** properties)
{
    return KDefaultAssociators(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_AccountCapabilities_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole,
        properties);
}

static CMPIStatus Linux_AccountCapabilitiesAssociatorNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole)
{
    return KDefaultAssociatorNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_AccountCapabilities_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole);
}

static CMPIStatus Linux_AccountCapabilitiesReferences(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role,
    const char** properties)
{
    return KDefaultReferences(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_AccountCapabilities_ClassName,
        assocClass,
        role,
        properties);
}

static CMPIStatus Linux_AccountCapabilitiesReferenceNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role)
{
    return KDefaultReferenceNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_AccountCapabilities_ClassName,
        assocClass,
        role);
}

CMInstanceMIStub( 
    Linux_AccountCapabilities,
    Linux_AccountCapabilities,
    _cb,
    Linux_AccountCapabilitiesInitialize())

CMAssociationMIStub( 
    Linux_AccountCapabilities,
    Linux_AccountCapabilities,
    _cb,
    Linux_AccountCapabilitiesInitialize())

KONKRET_REGISTRATION(
    "root/cimv2",
    "Linux_AccountCapabilities",
    "Linux_AccountCapabilities",
    "instance association");
