#include <konkret/konkret.h>
#include "Linux_AccountOnSystem.h"
#include "Linux_Account.h"
#include "Linux_ComputerSystem.h"

#include <glib.h>

#include <libuser/entity.h>
#include <libuser/user.h>

#include "aux_lu.h"
#include "globals.h"

static const CMPIBroker* _cb;

static void Linux_AccountOnSystemInitialize()
{
}

static CMPIStatus Linux_AccountOnSystemCleanup( 
    CMPIInstanceMI* mi,
    const CMPIContext* cc, 
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Linux_AccountOnSystemEnumInstanceNames( 
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus Linux_AccountOnSystemEnumInstances( 
    CMPIInstanceMI* mi,
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const char** properties) 
{
    Linux_AccountRef laref;
    Linux_ComputerSystemRef lcsref;
    Linux_AccountOnSystem laos;

    struct lu_context *luc = NULL;
    struct lu_error *error = NULL;
    GPtrArray *accounts = NULL;
    struct lu_ent *lue = NULL;

    size_t i;
    const char *nameSpace = KNameSpace(cop);
    const char *hostname = get_system_name();

    Linux_ComputerSystemRef_Init(&lcsref, _cb, nameSpace);
    Linux_ComputerSystemRef_Set_Name(&lcsref, hostname);
    Linux_ComputerSystemRef_Set_CreationClassName(&lcsref,
      Linux_ComputerSystem_ClassName);

    luc = lu_start(NULL, lu_user, NULL, NULL, lu_prompt_console_quiet, NULL, &error);
    if (!luc)
      {
        fprintf(stderr, "Error initializing: %s\n", lu_strerror(error));
        exit(1);
      }

    accounts = lu_users_enumerate_full(luc, "*", &error);
    for (i = 0;  (accounts != NULL) && (i < accounts->len); i++)
      {
        lue = g_ptr_array_index(accounts, i);

        Linux_AccountRef_Init(&laref, _cb, nameSpace);
        Linux_AccountRef_Set_SystemCreationClassName(&laref, SCCLASSNAME);
        Linux_AccountRef_Set_SystemName(&laref, hostname);
        Linux_AccountRef_Set_CreationClassName(&laref,
          Linux_Account_ClassName);
        Linux_AccountRef_Set_Name(&laref, aux_lu_get_str(lue, LU_USERNAME));

        Linux_AccountOnSystem_Init(&laos, _cb, nameSpace);
        Linux_AccountOnSystem_Set_GroupComponent(&laos, &lcsref);
        Linux_AccountOnSystem_Set_PartComponent(&laos, &laref);

        KReturnInstance(cr, laos);
        lu_ent_free(lue);
      } /* for */

    if (accounts)
      {
        g_ptr_array_free(accounts, TRUE);
      }

    lu_end(luc);
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Linux_AccountOnSystemGetInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc,
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const char** properties) 
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus Linux_AccountOnSystemCreateInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const CMPIInstance* ci) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_AccountOnSystemModifyInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop,
    const CMPIInstance* ci, 
    const char**properties) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_AccountOnSystemDeleteInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_AccountOnSystemExecQuery(
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const char* lang, 
    const char* query) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_AccountOnSystemAssociationCleanup( 
    CMPIAssociationMI* mi,
    const CMPIContext* cc, 
    CMPIBoolean term) 
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Linux_AccountOnSystemAssociators(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole,
    const char** properties)
{
    return KDefaultAssociators(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_AccountOnSystem_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole,
        properties);
}

static CMPIStatus Linux_AccountOnSystemAssociatorNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole)
{
    return KDefaultAssociatorNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_AccountOnSystem_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole);
}

static CMPIStatus Linux_AccountOnSystemReferences(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role,
    const char** properties)
{
    return KDefaultReferences(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_AccountOnSystem_ClassName,
        assocClass,
        role,
        properties);
}

static CMPIStatus Linux_AccountOnSystemReferenceNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role)
{
    return KDefaultReferenceNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_AccountOnSystem_ClassName,
        assocClass,
        role);
}

CMInstanceMIStub( 
    Linux_AccountOnSystem,
    Linux_AccountOnSystem,
    _cb,
    Linux_AccountOnSystemInitialize())

CMAssociationMIStub( 
    Linux_AccountOnSystem,
    Linux_AccountOnSystem,
    _cb,
    Linux_AccountOnSystemInitialize())

KONKRET_REGISTRATION(
    "root/cimv2",
    "Linux_AccountOnSystem",
    "Linux_AccountOnSystem",
    "instance association");
