#include <konkret/konkret.h>
#include "Linux_OwningGroup.h"
#include "Linux_ComputerSystem.h"
#include "Linux_Group.h"

#include "globals.h"
#include "aux_lu.h"

#include <libuser/user.h>
#include <libuser/entity.h>

static const CMPIBroker* _cb;

static void Linux_OwningGroupInitialize()
{
}

static CMPIStatus Linux_OwningGroupCleanup( 
    CMPIInstanceMI* mi,
    const CMPIContext* cc, 
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Linux_OwningGroupEnumInstanceNames( 
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus Linux_OwningGroupEnumInstances( 
    CMPIInstanceMI* mi,
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const char** properties) 
{
    Linux_ComputerSystemRef lcsref;
    Linux_GroupRef lgref;
    Linux_OwningGroup log;

    struct lu_context *luc = NULL;
    struct lu_error *error = NULL;
    GPtrArray *groups = NULL;
    struct lu_ent *lue = NULL;

    size_t i;
    const char *nameSpace = KNameSpace(cop);
    const char *hostname = get_system_name();

    Linux_ComputerSystemRef_Init(&lcsref, _cb, nameSpace);
    Linux_ComputerSystemRef_Set_Name(&lcsref, hostname);
    Linux_ComputerSystemRef_Set_CreationClassName(&lcsref,
      Linux_ComputerSystem_ClassName);

    luc = lu_start(NULL, lu_group, NULL, NULL, lu_prompt_console_quiet, NULL, &error);
    if (!luc)
      {
        fprintf(stderr, "Error initializing: %s\n", lu_strerror(error));
        exit(1);
      }
    groups = lu_groups_enumerate_full(luc, "*", &error);
    for (i = 0;  (groups != NULL) && (i < groups->len); i++)
      {
        lue = g_ptr_array_index(groups, i);

        Linux_GroupRef_Init(&lgref, _cb, nameSpace);
        Linux_GroupRef_Set_Name(&lgref, aux_lu_get_str(lue, LU_GROUPNAME));
        Linux_GroupRef_Set_CreationClassName(&lgref, Linux_Group_ClassName);

        Linux_OwningGroup_Init(&log, _cb, nameSpace);
        Linux_OwningGroup_Set_OwningElement(&log, &lcsref);
        Linux_OwningGroup_Set_OwnedElement(&log, &lgref);

        KReturnInstance(cr, log);
        lu_ent_free(lue);
      } /* for */

    if (groups)
      {
        g_ptr_array_free(groups, TRUE);
      }

    lu_end(luc);

    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Linux_OwningGroupGetInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc,
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const char** properties) 
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus Linux_OwningGroupCreateInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const CMPIInstance* ci) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_OwningGroupModifyInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop,
    const CMPIInstance* ci, 
    const char**properties) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_OwningGroupDeleteInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_OwningGroupExecQuery(
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const char* lang, 
    const char* query) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_OwningGroupAssociationCleanup( 
    CMPIAssociationMI* mi,
    const CMPIContext* cc, 
    CMPIBoolean term) 
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Linux_OwningGroupAssociators(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole,
    const char** properties)
{
    return KDefaultAssociators(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_OwningGroup_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole,
        properties);
}

static CMPIStatus Linux_OwningGroupAssociatorNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole)
{
    return KDefaultAssociatorNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_OwningGroup_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole);
}

static CMPIStatus Linux_OwningGroupReferences(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role,
    const char** properties)
{
    return KDefaultReferences(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_OwningGroup_ClassName,
        assocClass,
        role,
        properties);
}

static CMPIStatus Linux_OwningGroupReferenceNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role)
{
    return KDefaultReferenceNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_OwningGroup_ClassName,
        assocClass,
        role);
}

CMInstanceMIStub( 
    Linux_OwningGroup,
    Linux_OwningGroup,
    _cb,
    Linux_OwningGroupInitialize())

CMAssociationMIStub( 
    Linux_OwningGroup,
    Linux_OwningGroup,
    _cb,
    Linux_OwningGroupInitialize())

KONKRET_REGISTRATION(
    "root/cimv2",
    "Linux_OwningGroup",
    "Linux_OwningGroup",
    "instance association");
